;***************************************************************
; project name:    PCI Device Enumeration
; target platform: KolibriOS and MenuetOS
; compiler:        flat assmebler 1.63.2
; version:         1.30
; last update:     11(th) May 2006
; maintained by:   Jason Delozier (cordata51)
; e-mail:          cordata51@hotmail.com
; project site:    http://www.asmfreaks.com/menuetos
;***************************************************************
;Summary: This program will attempt to scan the PCI Bus
;        and display basic information about each device
;        connected to the PCI Bus.
;***************************************************************
;HISTORY:
;keep dates in european format (dd/mm/yyyy), please
; '!' means big changes
; to-do:
;PCI version should be normalized (0210->02.10)
;----------------------------------------------------------------
;1.30: PCIDEV   11/05/2006
;Author:    Sergey Kuzmin aka Wildwest <kuzmin_serg@list.ru>
;Features:  
;           added 
;                * 3 new vendor id's (ESS from Madis Kalme and 2 id's
;                  forgotten from 1.15 release: Broadcom -SiByte and Chaintech Comp.)
;			changed
;			     * I don't know why other devs (Jason or Victor) changed window style 
;                  to old ugly one, so I changed it back to skinned type 3.
;                * the same goes to the use of macroc.inc - it is enabled again.
;           deleted 
;                * there is no more label "PREVIOUSVERSIONLIST" - id's moved to the
;                  appropriate parts of global list.
;----------------------------------------------------------------
;1.29: PCIDEV   30/04/2006
;Author:    Jason Delozier <cordata51@hotmail.com>
;Features:  
;           fixed 
;                 * ! bug that would not allow devices with device
;                   numbers > 16 to be displayed.  
;           added 
;                 * ! another heading called "FNC" (function) which allows 
;                   the multipurpose Device/Function varible to be split and 
;                   displayed to the user properly.  
;                 * horizontal bars to display for easier reading.
;           optimized 
;                 * vendor/description search routines for speed and space.
;----------------------------------------------------------------
;1.25: PCIDEV   02/10/2005
;Author:    Sergey Kuzmin aka Wildwest <kuzmin_serg@list.ru>
;Features:  
;            changed
;					* ! Description is based on Class and SubClass 
;                     now (PCI 3.0). The Names of Classes and SubClasses 
;                     are in the end of Vendors.inc
;            deleted 
;                  * label "Descriptions" (names of Classes)
;           
;----------------------------------------------------------------
;1.20: PCIDEV   16/08/2005
;Author:    Victor Alberto Gil Hanla a.k.a. vhanla <vhanla@gmail.com>
;Features:  
;           added 
;                  * ! many vendor lists (865)
;           deleted
;                  * previous version's list 
;           changed
;                  * previous Company Name searching and printing 
;----------------------------------------------------------------
;1.15: PCIDEV   03/06/2005
;Author:    Sergey Kuzmin aka Wildwest <kuzmin_serg@list.ru>
;Features:  
;           added 
;				   * quantity of devices,
;				   * ! detection of Company Name based on Vendor ID,
;				   * database of VenID (35 ID's),
;				   * macros.inc for smaller size,
;           changed
;				   * interface+(skinned window),
;				   * VenID before DevID in 'table'(was DevID before VenID)
;----------------------------------------------------------------
;1.0: PCIDEV    30/01/2005
;Author:    Jason Delozier
;Features:  
;          able to 
;                 * detect PCI version, 
;                 * quantity of PCI buses,
;                 * Vendor&Device ID for appropriate Device on Bus;
;                 * detect Revision, Class and Subclass of Device,
;                 * and make Description based on Class
;-------------------------------------------------------------
use32

        org    0x0

        db     'MENUET01'; 8 byte id
        dd     0x01      ; header version
        dd     START     ; start of code
        dd     I_END     ; size of image
        dd     0xFFFF    ; memory for app = 64 KB
        dd     I_END     ; esp
        dd     0x0 , 0x0 ; I_Param , I_Icon


include 'macros.inc'
include 'VENDORS.INC'

START:    ; start of execution
     call draw_window

still:
    mov  eax,10   ; wait here for event
    int  0x40

    cmp  eax,1   ; redraw request ?
    je  red
    cmp  eax,2   ; key in buffer ?
    je  key
    cmp  eax,3   ; button in buffer ?
    je  button

    jmp  still

  red:    ; redraw
    mov eax, 9
    mov ebx, Proc_Info
    mov ecx, -1
    int 0x40
    mov eax,[Proc_Info+34]
    mov [Form+2], ax
    mov eax,[Proc_Info+38]
    mov [Form+6],ax
    mov eax,[Proc_Info+42]
    mov [Form],ax
    mov eax,[Proc_Info+46]
    mov [Form+4] ,ax

    call draw_window
    jmp  still
  key:    ; key
    mov  eax,2   ; just read it and ignore
    int  0x40
    jmp  still
  button:   ; button
    mov  eax,17   ; get id
    int  0x40

    cmp  ah,1   ; button id=1 ?
    jne  noclose

    mov  eax,-1   ; close this program
    int  0x40
  noclose:
    jmp  still




;   *********************************************
;   *******  WINDOW DEFINITIONS AND DRAW ********
;   *********************************************
Form:
   dw 780 ;window width
   dw 100 ;window x start
   dw 420 ;window height
   dw 100;window y start



draw_window:

    mov [total],0
	
    mov  eax,12      ; function 12:tell os about windowdraw
    mov  ebx,1       ; 1, start of draw
    int  0x40
    ; DRAW WINDOW
    mov  eax,0                     ; function 0 : define and draw window
    mov  ebx, dword [Form]
    mov  ecx, dword [Form+4]
    mov  edx,0x03ffffff            ; color of work area RRGGBB,8->color gl
    mov  esi,0x805080d0            ; color of grab bar  RRGGBB,8->color gl
    mov  edi,0x005080d0            ; color of frames    RRGGBB
    int  0x40
    ; WINDOW LABEL
    mov  eax,4                     ; function 4 : write text to window
    mov  ebx,8*65536+8             ; [x start] *65536 + [y start]
    mov  ecx,0x10ddeeff            ; font 1 & color ( 0xF0RRGGBB )
    mov  edx,labelt                ; pointer to text beginning
    mov  esi,labellen-labelt       ; text length
    int  0x40

    ; CLOSE BUTTON    - NO NEED IN BUTTON IN CASE OF WINDOW TYPE 3
 ;   mov  eax,8                     ; function 8 : define and draw button
 ;   mov bx, word [Form]            ;
 ;   sub bx, 19                     ;
 ;   shl ebx, 16                    ;
 ;   add ebx, 12                    ;
 ;   mov  ecx,5*65536+12            ; [y start] *65536 + [y size]
 ;   mov  edx,1                     ; button id
 ;   mov  esi,0x6688dd              ; button color RRGGBB
 ;   int  0x40                      ;

    ;draw captions to window
    mov ebx, 20*65536+25    ;Draw work format
    mov ecx, 0x224466       ;
    mov edx, PCIWin         ;
    mov esi, 106            ;lenght of line
newline:                    ;
    mov eax, 4              ;
    int 0x40                ;
    add ebx, 10             ;
    add edx, 106            ; lenght of line
    cmp [edx], byte 'x'
    jne newline

    ;Insert bars
    mov eax, 13
    mov ebx, 18
    shl ebx, 16
    mov bx, [Form]
    sub bx, 32
    mov ecx, 109*65536+10
    mov edx, 0xb6b6b6
   again:
    int 0x40
    shr ecx, 16
    add ecx, 36
    cmp cx,word [Form+4]
    jae nomo
    sub ecx, 16
    shl ecx, 16
    add ecx, 10
    jmp again
   nomo:

    ;start PCI stuff
    call Get_PCI_Info

    mov cx, [PCI_Version]
    mov eax, 47
    mov ebx, 0x00040100
    mov esi, 0x00000000
    mov edx, 110*65536+45
    int 0x40

    mov cl, [PCI_LastBus]
    mov eax, 47
    mov ebx, 0x00020100
    mov esi, 0x00000000
    mov edx, 110*65536+55
    int 0x40

    call scan_Pci

    xor ecx, ecx
    mov cl, [total]
    mov eax, 47
    mov ebx, 0x00020000
    mov esi, 0x00000000
    mov edx, 150*65536+65
    int 0x40


    mov  eax,12      ; function 12:tell os about windowdraw
    mov  ebx,2      ; 2, end of draw
    int  0x40

    ret
;   ***********************************************
;   *******  END WINDOW DEFINITIONS & DRAW  *******
;   ***********************************************

Get_PCI_Info:
    mov eax, 62
    xor ebx, ebx
    int 0x40
    mov [PCI_Version], ax
    mov eax, 62
    mov ebx, 1
    int 0x40
    mov [PCI_LastBus], al
ret



scan_Pci:
   cmp [PCI_LastBus],0xff
   jne Pci_Exists
   ret
Pci_Exists:
   mov [V_Bus], 0   ;reset varibles
   mov [V_Dev], 0   ;
   mov edx,  20*65536+110 ;start write position

Start_Enum:
   mov bl, 6      ;get Device ID / Vendor ID
   mov bh, [V_Bus]
   mov ch, [V_Dev]
   mov cl, 0
   mov eax, 62
   int 0x40

   cmp ax, 0      ;check next slot if nothing exists
   je nextDev
   cmp ax, 0xffff
   je nextDev

   mov [PCI_Vendor], ax
   shr eax, 16
   mov [PCI_Device], ax

   mov eax, 62       ;PCI Sys Function
   mov bl, 4         ;Read config byte
   mov bh, [V_Bus]   ;Bus #
   mov ch, [V_Dev]   ;Device # on bus
   mov cl, 0x08      ;Register to read (Get Revision)
   int 0x40          ;Read it
   mov [PCI_Rev], al ;Save it

   mov eax, 62            ;PCI Sys Function
   mov cl, 0x0b           ;Register to read (Get class)
   int 0x40               ;Read it
   mov [PCI_Class], al    ;Save it

   mov eax, 62            ;PCI Sys Function
   mov cl, 0x0a           ;Register to read (Get Subclass)
   int 0x40               ;Read it
   mov [PCI_SubClass], al ;Save it

   call Print_New_Device

nextDev:
   inc [V_Dev]
   jnz Start_Enum         ;jump we reach zero
                          ;(used to be JNO which caused bug!!! 30-4-2006, JMD)

   mov [V_Dev], 0
   inc [V_Bus]
   mov al, [PCI_LastBus]

   cmp byte [V_Bus], al
   jbe Start_Enum
ret

Print_New_Device:
   inc [total]
   xor eax, eax
   mov al,[total]
   add al, 120
   shl  eax, 4
   mov [Form+4], ax   ; 120+16*[total]
   xor eax, eax

   mov eax, 47           ;Write number to screen
   mov ebx,0x00040100    ;4 byte number, print in hexidecimal
   mov esi, 0x00000000   ;Color of text
   xor ecx,ecx
   mov cx,[PCI_Vendor]   ;Pointer to number to be written
   int 0x40              ;Write it

   push dx               ;Calculate position
   mov dx, 54            ;
   shl edx, 16           ;
   pop dx                ;
   mov cx, [PCI_Device]  ;Write Vendor ID
   int 0x40              ;

   mov ebx, 0x00020100   ;2 byte number, in hexidecimal format
   push dx               ;move write cursor
   mov dx, 98            ;
   shl edx, 16           ;
   pop dx                ;
   mov cl, [V_Bus]       ;write bus number
   int 0x40              ;

   push dx               ;move write cursor
   mov dx, 128           ;
   shl edx, 16           ;
   pop dx                ;
   mov cl, [V_Dev]       ;Write device number
   shr cl, 3             ;
   int 0x40              ;

   push dx               ;move write cursor
   mov dx, 155           ;
   shl edx, 16           ;
   pop dx                ;
   mov cl, [V_Dev]       ;Write Function number
   and cl, 7             ;
   int 0x40              ;



   push dx               ;move write cursor
   mov dx, 179           ;
   shl edx, 16           ;
   pop dx                ;
   mov cl, [PCI_Rev]     ;Draw Revision to screen
   int 0x40              ;

   push dx               ;move write cursor
   mov dx, 215           ;
   shl edx, 16           ;
   pop dx                ;
   mov cl, [PCI_Class]   ;Draw Class to screen
   int 0x40              ;

   push dx               ;move write curson
   mov dx, 266           ;
   shl edx, 16           ;
   pop dx                ;
   mov cl, [PCI_SubClass];Draw Sub Class to screen
   int 0x40              ;


     ;Write Names
   mov bx, 310           ;X-position of text on screen
   shl ebx, 16           ;
   mov bx, dx            ;Current Y position


;**********************************************************
;Prints the Vendor's Name based on Vendor ID
;
; modified part by vhanla (I know it is not a fastest way to search)
; it needs optimization... HELP this project!
;
; Modified on 30-04-2006 by JMD for size
;-----------------------------------------------------------------------------
    ;first determine which list to find the vendor in
    mov ax, word [PCI_Vendor]
    mov ecx, 255           ;# vendors in most lists
    cmp [PCI_Vendor],4800  ;Check if Vendor's value is less than this number
    jae next1              ;if it is less, let's continue, or jump to next1
      mov edx, _FIRSTPART  ;select this list
      jmp rep1             ;start searching list
  next1:                   ;
    cmp [PCI_Vendor],5314  ;same thing happening here as above^
    jae next2              ;
      mov edx, _SECONDPART ;
      jmp rep1             ;
  next2:                   ;
    cmp [PCI_Vendor],5574  ;
    jae next3              ;
      mov edx,_THIRDPART   ;
      jmp rep1             ;
  next3:                   ;
    cmp [PCI_Vendor],65535 ;the last in a WORD
    jae rep1              ;
      mov ecx, 110         ;only 110 vendors in this list
      mov edx, _FOURTHPART ;
      jmp rep1             ;



 rep1:
   cmp ax,word[edx+50]       ;are Vendor ID's the same?
   je  ex                    ;if so jump to print the description to screen
   add edx, 52               ;if not put us at start of next description
   dec ecx                   ;one less description in list
   jnz rep1                  ;was it our last?
   mov edx, _UNKNOWN         ;if so we dont know this Vendor

 ex:
  ;lets print the vendor Name
   xor ecx, ecx              ;font color
   mov eax,4                 ;OS CMD
   mov esi,50                ;Length of text
   int 0x40                  ;Print the text
;------------------------------------------------------------------



;------------------------------------------------------------------
;Get description based on Class/Subclass
   cmp [PCI_Class], 11h        ;we only know of 17 classes
   ja  endd                    ;if its more then, its unknown to us, so jump
   movzx eax, byte [PCI_Class] ;load our class
   shl eax, 3                  ;multiply for jump table
   mov ecx, [ClassList+eax+4]  ;number of descriptions for this class
   mov edx, [ClassList+eax]    ;start of description list for class
   mov ax, word[PCI_SubClass]
 repu1:
   cmp ax,word[edx+32]         ;are subclasses the same?
   je  endd                    ;if so jump to print the description to screen
   add edx, 34                 ;if not put us at start of next description
   dec ecx                     ;one less description in list
   jnz repu1                   ;was it our last?
   mov edx,_UNKNOWND           ;if so its unknown device

endd:
   push bx         ;
   mov  bx, 560    ;X-position of text on screen
   shl ebx, 16     ;
   pop  bx         ;
   xor ecx, ecx    ;color of text
   mov eax,4       ;
   mov esi,32      ;
   int 0x40        ;

   mov  dx, 20     ;Starting Y coordinate
   shl edx, 16     ;
   mov  dx, bx     ;
   add edx, 10     ;Next line down
ret

ClassList:
 dd Class0 ,  2, Class1 , 8, Class2,  8, Class3, 4
 dd Class4 ,  4, Class5 , 3, Class6, 12, Class7, 7
 dd Class8 ,  8, Class9 , 6, ClassA,  2, ClassB, 7
 dd ClassC , 10, ClassD , 8, ClassE,  1, ClassF, 4
 dd Class10,  3, Class11, 5
;------------------------------------------------------------------



; DATA AREA

labelt:
db 'PCI Device Enumeration v 1.30 by J. Delozier, S. Kuzmin and V. Hanla'
 labellen:

PCIWin:
db 'Please remember to enable PCI Access to Applications in Setup Menu.'
db '                                       '
db '                                                                   '
db '                                       '
db 'PCI Version  =                                                     '
db '                                       '
db 'Last PCI Bus =                                                     '
db '                                       '
db 'Quantity of devices =                                              '
db '                                       '
db '                                                                   '
db '                                       '
db 'VenID DevID Bus# Dev# Fnc Rev  Class  Subclass                 Comp'
db 'any                      Description   '
db '----- ----- ---- ---- --- ---  -----  -------- --------------------'
db '---------------------- ----------------'
db 'x'

total        db 0
V_Bus        db 0
V_Dev        db 0
PCI_Version  dw 0
PCI_LastBus  db 0
PCI_Device   dw 0
PCI_Vendor   dw 0
PCI_Bus      db 0
PCI_Dev      db 0
PCI_Rev      db 0
PCI_Class    db 0
PCI_SubClass db 0


J db 0        ;if this isnt here then the PROC_INFO screws up subclass varible
              ;Is this a Kernal issue or did i mess something up in here?
Proc_Info:
 times 1024 db 0

I_END: